@@ -150,17 +150,14 @@ def get_available_cores():
         return 1
 
 
-EPILOG_TEXT = (
-    """
+EPILOG_TEXT = f"""
 available shortcuts:
   S - skip execution of the current pass
   D - toggle --print-diff option
 
 For bug reporting instructions, please use:
-%s
+{CVise.Info.PACKAGE_URL}
 """
-    % CVise.Info.PACKAGE_URL
-)
 
 if __name__ == '__main__':
     parser = argparse.ArgumentParser(
@@ -312,7 +309,7 @@ if __name__ == '__main__':
         '--version',
         action='version',
         version=CVise.Info.PACKAGE_STRING
-        + (' (%s)' % CVise.Info.GIT_VERSION if CVise.Info.GIT_VERSION != 'unknown' else ''),
+        + (f' ({CVise.Info.GIT_VERSION})' if CVise.Info.GIT_VERSION != 'unknown' else ''),
     )
     parser.add_argument(
         '--commands',
@@ -337,6 +334,12 @@ if __name__ == '__main__':
         help='Executable to check interestingness of test cases',
     )
     parser.add_argument('test_cases', metavar='TEST_CASE', nargs='+', help='Test cases')
+    parser.add_argument(
+        '--stopping-threshold',
+        default=1.0,
+        type=float,
+        help='CVise will stop reducing a test case once it has reduced by this fraction of its original size.  Between 0.0 and 1.0.',
+    )
 
     args = parser.parse_args()
 
@@ -354,15 +357,18 @@ if __name__ == '__main__':
     else:
         log_config['level'] = getattr(logging, args.log_level.upper())
 
-    if args.log_file is not None:
-        log_config['filename'] = args.log_file
-
-    logging.basicConfig(**log_config)
-    syslog = logging.StreamHandler()
+    logging.getLogger().setLevel(log_config['level'])
     formatter = DeltaTimeFormatter(log_format)
-    syslog.setFormatter(formatter)
-    logging.getLogger().handlers = []
-    logging.getLogger().addHandler(syslog)
+    root_logger = logging.getLogger()
+
+    if args.log_file is not None:
+        file_handler = logging.FileHandler(args.log_file)
+        file_handler.setFormatter(formatter)
+        root_logger.addHandler(file_handler)
+    else:
+        syslog = logging.StreamHandler()
+        syslog.setFormatter(formatter)
+        root_logger.addHandler(syslog)
 
     pass_options = set()
 
@@ -441,7 +447,7 @@ if __name__ == '__main__':
             script.write(f'#!/usr/bin/env {args.shell}\n\n')
             script.write(args.commands + '\n')
         os.chmod(script.name, 0o744)
-        logging.info('Using temporary interestingness test: %s' % script.name)
+        logging.info(f'Using temporary interestingness test: {script.name}')
         args.interestingness_test = script.name
 
     test_manager = testing.TestManager(
@@ -461,6 +467,7 @@ if __name__ == '__main__':
         args.also_interesting,
         args.start_with_pass,
         args.skip_after_n_transforms,
+        args.stopping_threshold,
     )
 
     reducer = CVise(test_manager, args.skip_interestingness_test_check)
@@ -477,43 +484,44 @@ if __name__ == '__main__':
         print(err)
     else:
         time_stop = time.monotonic()
-        print('===< PASS statistics >===')
-        print(
-            '  %-60s %8s %8s %8s %8s %15s'
-            % (
-                'pass name',
-                'time (s)',
-                'time (%)',
-                'worked',
-                'failed',
-                'total executed',
-            )
-        )
-
-        for pass_name, pass_data in pass_statistic.sorted_results:
-            print(
-                '  %-60s %8.2f %8.2f %8d %8d %15d'
+        with open(args.log_file, 'a') if args.log_file else sys.stderr as fs:
+            fs.write('===< PASS statistics >===\n')
+            fs.write(
+                '  %-60s %8s %8s %8s %8s %15s\n'
                 % (
-                    pass_name,
-                    pass_data.total_seconds,
-                    100.0 * pass_data.total_seconds / (time_stop - time_start),
-                    pass_data.worked,
-                    pass_data.failed,
-                    pass_data.totally_executed,
+                    'pass name',
+                    'time (s)',
+                    'time (%)',
+                    'worked',
+                    'failed',
+                    'total executed',
                 )
             )
-        print()
 
-        if not args.no_timing:
-            print(f'Runtime: {round(time_stop - time_start)} seconds')
+            for pass_name, pass_data in pass_statistic.sorted_results:
+                fs.write(
+                    '  %-60s %8.2f %8.2f %8d %8d %15d\n'
+                    % (
+                        pass_name,
+                        pass_data.total_seconds,
+                        100.0 * pass_data.total_seconds / (time_stop - time_start),
+                        pass_data.worked,
+                        pass_data.failed,
+                        pass_data.totally_executed,
+                    )
+                )
+            fs.write('\n')
 
-        print('Reduced test-cases:\n')
-        for test_case in sorted(test_manager.test_cases):
-            if misc.is_readable_file(test_case):
-                print(f'--- {test_case} ---')
-                with open(test_case) as test_case_file:
-                    print(test_case_file.read())
-        if script:
-            os.unlink(script.name)
+            if not args.no_timing:
+                fs.write(f'Runtime: {round(time_stop - time_start)} seconds\n')
+
+            fs.write('Reduced test-cases:\n\n')
+            for test_case in sorted(test_manager.test_cases):
+                if misc.is_readable_file(test_case):
+                    print(f'--- {test_case} ---')
+                    with open(test_case) as test_case_file:
+                        fs.write(test_case_file.read() + '\n')
+            if script:
+                os.unlink(script.name)
 
     logging.shutdown()
