@@ -1,21 +1,22 @@
-from concurrent.futures import FIRST_COMPLETED, TimeoutError, wait
+from concurrent.futures import FIRST_COMPLETED, wait
 import difflib
 import filecmp
 import logging
 import math
 from multiprocessing import Manager
 import os
-import os.path
+from pathlib import Path
 import platform
 import shutil
 import subprocess
 import sys
 import tempfile
 import traceback
+import concurrent.futures
 
 from cvise.cvise import CVise
 from cvise.passes.abstract import PassResult, ProcessEventNotifier, ProcessEventType
-from cvise.utils.error import FolderInPathTestCaseError
+from cvise.utils.error import AbsolutePathTestCaseError
 from cvise.utils.error import InsaneTestCaseError
 from cvise.utils.error import InvalidInterestingnessTestError
 from cvise.utils.error import InvalidTestCaseError
@@ -32,7 +33,7 @@ MAX_PASS_INCREASEMENT_THRESHOLD = 3
 
 
 def rmfolder(name):
-    assert 'cvise' in name
+    assert 'cvise' in str(name)
     try:
         shutil.rmtree(name)
     except OSError:
@@ -40,10 +41,17 @@ def rmfolder(name):
 
 
 class TestEnvironment:
-    def __init__(self, state, order, test_script, folder, test_case,
-                 additional_files, transform, pid_queue=None):
-        self.test_case = None
-        self.additional_files = set()
+    def __init__(
+        self,
+        state,
+        order,
+        test_script,
+        folder,
+        test_case,
+        all_test_cases,
+        transform,
+        pid_queue=None,
+    ):
         self.state = state
         self.folder = folder
         self.base_size = None
@@ -53,52 +61,40 @@ class TestEnvironment:
         self.order = order
         self.transform = transform
         self.pid_queue = pid_queue
-        self.copy_files(test_case, additional_files)
         self.pwd = os.getcwd()
+        self.test_case = test_case
+        self.base_size = test_case.stat().st_size
+        self.all_test_cases = all_test_cases
 
-    def copy_files(self, test_case, additional_files):
-        if test_case is not None:
-            self.test_case = os.path.basename(test_case)
-            shutil.copy(test_case, self.folder)
-            self.base_size = os.path.getsize(test_case)
-
-        for f in additional_files:
-            self.additional_files.add(os.path.basename(f))
-            shutil.copy(f, self.folder)
+        # Copy files to the created folder
+        for test_case in all_test_cases:
+            (self.folder / test_case.parent).mkdir(parents=True, exist_ok=True)
+            shutil.copy2(test_case, self.folder / test_case.parent)
 
     @property
     def size_improvement(self):
-        if self.base_size is None:
-            return None
-        else:
-            return (self.base_size - os.path.getsize(self.test_case_path))
+        return self.base_size - self.test_case_path.stat().st_size
 
     @property
     def test_case_path(self):
-        return os.path.join(self.folder, self.test_case)
-
-    @property
-    def additional_files_paths(self):
-        return [os.path.join(self.folder, f) for f in self.additional_files]
+        return self.folder / self.test_case
 
     @property
     def success(self):
         return self.result == PassResult.OK and self.exitcode == 0
 
     def dump(self, dst):
-        if self.test_case is not None:
-            shutil.copy(self.test_case_path, dst)
-
-        for f in self.additional_files:
-            shutil.copy(f, dst)
+        for f in self.all_test_cases:
+            shutil.copy(self.folder / f, dst)
 
         shutil.copy(self.test_script, dst)
 
     def run(self):
         try:
             # transform by state
-            (result, self.state) = self.transform(self.test_case_path, self.state,
-                                                  ProcessEventNotifier(self.pid_queue))
+            (result, self.state) = self.transform(
+                str(self.test_case_path), self.state, ProcessEventNotifier(self.pid_queue)
+            )
             self.result = result
             if self.result != PassResult.OK:
                 return self
@@ -117,7 +113,9 @@ class TestEnvironment:
     def run_test(self, verbose):
         try:
             os.chdir(self.folder)
-            stdout, stderr, returncode = ProcessEventNotifier(self.pid_queue).run_process(self.test_script, shell=True)
+            stdout, stderr, returncode = ProcessEventNotifier(self.pid_queue).run_process(
+                str(self.test_script), shell=True
+            )
             if verbose and returncode != 0:
                 logging.debug('stdout:\n' + stdout)
                 logging.debug('stderr:\n' + stderr)
@@ -133,10 +131,27 @@ class TestManager:
     MAX_EXTRA_DIRS = 25000
     TEMP_PREFIX = 'cvise-'
 
-    def __init__(self, pass_statistic, test_script, timeout, save_temps, test_cases, parallel_tests,
-                 no_cache, skip_key_off, silent_pass_bug, die_on_pass_bug, print_diff, max_improvement,
-                 no_give_up, also_interesting, start_with_pass, skip_after_n_transforms):
-        self.test_script = os.path.abspath(test_script)
+    def __init__(
+        self,
+        pass_statistic,
+        test_script,
+        timeout,
+        save_temps,
+        test_cases,
+        parallel_tests,
+        no_cache,
+        skip_key_off,
+        silent_pass_bug,
+        die_on_pass_bug,
+        print_diff,
+        max_improvement,
+        no_give_up,
+        also_interesting,
+        start_with_pass,
+        skip_after_n_transforms,
+        stopping_threshold,
+    ):
+        self.test_script = Path(test_script).absolute()
         self.timeout = timeout
         self.save_temps = save_temps
         self.pass_statistic = pass_statistic
@@ -153,14 +168,15 @@ class TestManager:
         self.also_interesting = also_interesting
         self.start_with_pass = start_with_pass
         self.skip_after_n_transforms = skip_after_n_transforms
+        self.stopping_threshold = stopping_threshold
 
         for test_case in test_cases:
+            test_case = Path(test_case)
+            self.test_cases_modes[test_case] = test_case.stat().st_mode
             self.check_file_permissions(test_case, [os.F_OK, os.R_OK, os.W_OK], InvalidTestCaseError)
-            if os.path.split(test_case)[0]:
-                raise FolderInPathTestCaseError(test_case)
-            fullpath = os.path.abspath(test_case)
-            self.test_cases.add(fullpath)
-            self.test_cases_modes[fullpath] = os.stat(fullpath).st_mode
+            if test_case.parent.is_absolute():
+                raise AbsolutePathTestCaseError(test_case)
+            self.test_cases.add(test_case)
 
         self.orig_total_file_size = self.total_file_size
         self.cache = {}
@@ -168,14 +184,21 @@ class TestManager:
         if not self.is_valid_test(self.test_script):
             raise InvalidInterestingnessTestError(self.test_script)
 
-        self.use_colordiff = (sys.stdout.isatty() and
-                              subprocess.run('colordiff --version', shell=True,
-                                             stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL).returncode == 0)
+        self.use_colordiff = (
+            sys.stdout.isatty()
+            and subprocess.run(
+                'colordiff --version',
+                shell=True,
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL,
+            ).returncode
+            == 0
+        )
 
     def create_root(self):
         pass_name = str(self.current_pass).replace('::', '-')
         self.root = tempfile.mkdtemp(prefix=f'{self.TEMP_PREFIX}{pass_name}-')
-        logging.debug('Creating pass root folder: %s' % self.root)
+        logging.debug(f'Creating pass root folder: {self.root}')
 
     def remove_root(self):
         if not self.save_temps:
@@ -183,7 +206,7 @@ class TestManager:
 
     def restore_mode(self):
         for test_case in self.test_cases:
-            os.chmod(test_case, self.test_cases_modes[test_case])
+            test_case.chmod(self.test_cases_modes[test_case])
 
     @classmethod
     def is_valid_test(cls, test_script):
@@ -198,11 +221,11 @@ class TestManager:
 
     @property
     def sorted_test_cases(self):
-        return sorted(self.test_cases, key=os.path.getsize, reverse=True)
+        return sorted(self.test_cases, key=lambda x: x.stat().st_size, reverse=True)
 
     @staticmethod
     def get_file_size(files):
-        return sum(os.path.getsize(f) for f in files)
+        return sum(f.stat().st_size for f in files)
 
     @property
     def total_line_count(self):
@@ -219,9 +242,9 @@ class TestManager:
 
     def backup_test_cases(self):
         for f in self.test_cases:
-            orig_file = f'{f}.orig'
+            orig_file = Path(f'{f}.orig')
 
-            if not os.path.exists(orig_file):
+            if not orig_file.exists():
                 # Copy file and preserve attributes
                 shutil.copy2(f, orig_file)
 
@@ -240,14 +263,14 @@ class TestManager:
     def get_extra_dir(prefix, max_number):
         for i in range(0, max_number + 1):
             digits = int(round(math.log10(max_number), 0))
-            extra_dir = ('{0}{1:0' + str(digits) + 'd}').format(prefix, i)
+            extra_dir = Path(('{0}{1:0' + str(digits) + 'd}').format(prefix, i))
 
-            if not os.path.exists(extra_dir):
+            if not extra_dir.exists():
                 break
 
         # just bail if we've already created enough of these dirs, no need to
         # clutter things up even more...
-        if os.path.exists(extra_dir):
+        if extra_dir.exists():
             return None
 
         return extra_dir
@@ -263,17 +286,19 @@ class TestManager:
         if crash_dir is None:
             return False
 
-        os.mkdir(crash_dir)
+        crash_dir.mkdir()
         test_env.dump(crash_dir)
 
         if not self.die_on_pass_bug:
-            logging.debug(f'Please consider tarring up {crash_dir} and creating an issue at https://github.com/marxin/cvise/issues and we will try to fix the bug.')
+            logging.debug(
+                f'Please consider tarring up {crash_dir} and creating an issue at https://github.com/marxin/cvise/issues and we will try to fix the bug.'
+            )
 
-        with open(os.path.join(crash_dir, 'PASS_BUG_INFO.TXT'), mode='w') as info_file:
-            info_file.write('Package: %s\n' % CVise.Info.PACKAGE_STRING)
-            info_file.write('Git version: %s\n' % CVise.Info.GIT_VERSION)
-            info_file.write('LLVM version: %s\n' % CVise.Info.LLVM_VERSION)
-            info_file.write('System: %s\n' % str(platform.uname()))
+        with (crash_dir / 'PASS_BUG_INFO.TXT').open(mode='w') as info_file:
+            info_file.write(f'Package: {CVise.Info.PACKAGE_STRING}\n')
+            info_file.write(f'Git version: {CVise.Info.GIT_VERSION}\n')
+            info_file.write(f'LLVM version: {CVise.Info.LLVM_VERSION}\n')
+            info_file.write(f'System: {str(platform.uname())}\n')
             info_file.write(PassBugError.MSG.format(self.current_pass, problem, test_env.state, crash_dir))
 
         if self.die_on_pass_bug:
@@ -289,15 +314,15 @@ class TestManager:
         with open(changed_file) as f:
             changed_file_lines = f.readlines()
 
-        diffed_lines = difflib.unified_diff(orig_file_lines, changed_file_lines, orig_file, changed_file)
+        diffed_lines = difflib.unified_diff(orig_file_lines, changed_file_lines, str(orig_file), str(changed_file))
 
         return ''.join(diffed_lines)
 
     def check_sanity(self, verbose=False):
         logging.debug('perform sanity check... ')
 
-        folder = tempfile.mkdtemp(prefix=f'{self.TEMP_PREFIX}sanity-')
-        test_env = TestEnvironment(None, 0, self.test_script, folder, None, self.test_cases, None)
+        folder = Path(tempfile.mkdtemp(prefix=f'{self.TEMP_PREFIX}sanity-'))
+        test_env = TestEnvironment(None, 0, self.test_script, folder, list(self.test_cases)[0], self.test_cases, None)
         logging.debug(f'sanity check tmpdir = {test_env.folder}')
 
         returncode = test_env.run_test(verbose)
@@ -321,7 +346,7 @@ class TestManager:
 
     @classmethod
     def log_key_event(cls, event):
-        logging.info('****** %s ******' % event)
+        logging.info(f'****** {event} ******')
 
     def kill_pid_queue(self):
         active_pids = set()
@@ -338,7 +363,8 @@ class TestManager:
                 children.append(process)
                 for child in children:
                     try:
-                        child.terminate()
+                        # Terminate the process more reliability: https://github.com/marxin/cvise/issues/145
+                        child.kill()
                     except psutil.NoSuchProcess:
                         pass
             except psutil.NoSuchProcess:
@@ -366,7 +392,8 @@ class TestManager:
 
             if future.done():
                 if future.exception():
-                    if type(future.exception()) is TimeoutError:
+                    # starting with Python 3.11: concurrent.futures.TimeoutError == TimeoutError
+                    if type(future.exception()) in (TimeoutError, concurrent.futures.TimeoutError):
                         self.timeout_count += 1
                         logging.warning('Test timed out.')
                         self.save_extra_dir(self.temporary_folders[future])
@@ -379,8 +406,7 @@ class TestManager:
 
                 test_env = future.result()
                 if test_env.success:
-                    if (self.max_improvement is not None and
-                            test_env.size_improvement > self.max_improvement):
+                    if self.max_improvement is not None and test_env.size_improvement > self.max_improvement:
                         logging.debug(f'Too large improvement: {test_env.size_improvement} B')
                     else:
                         # Report bug if transform did not change the file
@@ -395,8 +421,7 @@ class TestManager:
                     self.pass_statistic.add_failure(self.current_pass)
                     if test_env.result == PassResult.OK:
                         assert test_env.exitcode
-                        if (self.also_interesting is not None and
-                                test_env.exitcode == self.also_interesting):
+                        if self.also_interesting is not None and test_env.exitcode == self.also_interesting:
                             self.save_extra_dir(test_env.test_case_path)
                     elif test_env.result == PassResult.STOP:
                         quit_loop = True
@@ -422,7 +447,8 @@ class TestManager:
                 test_env = future.result()
                 if test_env.success:
                     return test_env
-            except TimeoutError:
+            # starting with Python 3.11: concurrent.futures.TimeoutError == TimeoutError
+            except (TimeoutError, concurrent.futures.TimeoutError):
                 pass
         return None
 
@@ -448,10 +474,17 @@ class TestManager:
                     self.terminate_all(pool)
                     return success
 
-                folder = tempfile.mkdtemp(prefix=self.TEMP_PREFIX, dir=self.root)
-                test_env = TestEnvironment(self.state, order, self.test_script, folder,
-                                           self.current_test_case, self.test_cases ^ {self.current_test_case},
-                                           self.current_pass.transform, self.pid_queue)
+                folder = Path(tempfile.mkdtemp(prefix=self.TEMP_PREFIX, dir=self.root))
+                test_env = TestEnvironment(
+                    self.state,
+                    order,
+                    self.test_script,
+                    folder,
+                    self.current_test_case,
+                    self.test_cases,
+                    self.current_pass.transform,
+                    self.pid_queue,
+                )
                 future = pool.schedule(test_env.run, timeout=self.timeout)
                 self.temporary_folders[future] = folder
                 self.futures.append(future)
@@ -493,7 +526,7 @@ class TestManager:
         try:
             for test_case in self.sorted_test_cases:
                 self.current_test_case = test_case
-                starting_test_case_size = os.path.getsize(test_case)
+                starting_test_case_size = test_case.stat().st_size
                 success_count = 0
 
                 if self.get_file_size([test_case]) == 0:
@@ -503,8 +536,7 @@ class TestManager:
                     with open(test_case, mode='rb+') as tmp_file:
                         test_case_before_pass = tmp_file.read()
 
-                        if (pass_key in self.cache and
-                                test_case_before_pass in self.cache[pass_key]):
+                        if pass_key in self.cache and test_case_before_pass in self.cache[pass_key]:
                             tmp_file.seek(0)
                             tmp_file.truncate(0)
                             tmp_file.write(self.cache[pass_key][test_case_before_pass])
@@ -534,10 +566,12 @@ class TestManager:
                         success_count += 1
 
                     # if the file increases significantly, bail out the current pass
-                    test_case_size = os.path.getsize(self.current_test_case)
+                    test_case_size = self.current_test_case.stat().st_size
                     if test_case_size >= MAX_PASS_INCREASEMENT_THRESHOLD * starting_test_case_size:
-                        logging.info(f'skipping the rest of the pass (huge file increasement '
-                                     f'{MAX_PASS_INCREASEMENT_THRESHOLD * 100}%)')
+                        logging.info(
+                            f'skipping the rest of the pass (huge file increasement '
+                            f'{MAX_PASS_INCREASEMENT_THRESHOLD * 100}%)'
+                        )
                         break
 
                     self.release_folders()
@@ -546,8 +580,9 @@ class TestManager:
                         break
 
                     # skip after N transformations if requested
-                    if ((self.skip_after_n_transforms and success_count >= self.skip_after_n_transforms)
-                            or (self.current_pass.max_transforms and success_count >= self.current_pass.max_transforms)):
+                    if (self.skip_after_n_transforms and success_count >= self.skip_after_n_transforms) or (
+                        self.current_pass.max_transforms and success_count >= self.current_pass.max_transforms
+                    ):
                         logging.info(f'skipping after {success_count} successful transformations')
                         break
 
@@ -577,7 +612,9 @@ class TestManager:
         try:
             shutil.copy(test_env.test_case_path, self.current_test_case)
         except FileNotFoundError:
-            raise RuntimeError(f"Can't find {self.current_test_case} -- did your interestingness test move it?")
+            raise RuntimeError(
+                f"Can't find {self.current_test_case} -- did your interestingness test move it?"
+            ) from None
 
         self.state = self.current_pass.advance_on_success(test_env.test_case_path, test_env.state)
         self.pass_statistic.add_success(self.current_pass)
@@ -589,6 +626,6 @@ class TestManager:
         if self.total_line_count:
             notes.append(f'{self.total_line_count} lines')
         if len(self.test_cases) > 1:
-            notes.append(test_env.test_case)
+            notes.append(str(test_env.test_case))
 
         logging.info('(' + ', '.join(notes) + ')')
